import React, {Component} from 'react'
import './index.scss'
import {Paging} from '../common'

class Ad extends Component {
  state = {
    list: [],
    isEnter: false,
    activeIndexUI: 0
  }

  constructor(props) {
    super(props)
    this.wrapper = React.createRef()
    this.activeIndex = null
  }

  componentDidMount() {
    this.props.httpAgent.get('/sym/getAD.web').then(res => {
      if (res && res.resultCode === '000000' && res.result.appADList[0] && res.result.appADList[0].advertisementList) {
        const list = res.result.appADList[0].advertisementList.map(l => ({
          title: l.title,
          picUrl: l.picUrl,
          linkUrl: l.linkUrl,
        }));
        this.setState({list})
        setTimeout(this.start, 0)
      }
    })
  }

  start = () => {
    if (this.state.list.length <= 1) return
    if (this.activeIndex === null) {
      this.activeIndex = 1
      if (this.wrapper && this.wrapper.current) {
        this.wrapper.current.style.transform = `translate3d(${-483 * this.activeIndex}px,0,0)`
      }
    }
    clearTimeout(this.timer)
    this.timer = setTimeout(this.carousel, 5000)
  }

  end = () => {
    clearTimeout(this.timer)
  }

  onTransitionEnd = e => {
    if (this.state.isEnter) return
    this.start()
  }

  onMouseEnter = e => {
    this.setState({isEnter: true})
    this.end()
  }

  onMouseLeave = e => {
    this.setState({isEnter: false})
    this.start()
  }

  prev = () => {
    this.carousel(true)
  }

  next = () => {
    this.carousel(false)
  }

  toPage = activeIndex => {
    this.activeIndex = activeIndex + 1
    this.wrapper.current.className = 'anim'
    this.wrapper.current.style.transform = `translate3d(${-483 * this.activeIndex}px,0,0)`
    this.setState({activeIndexUI: activeIndex})
  }

  carousel = (direction = false) => {
    const {list} = this.state
    this.activeIndex = direction ? this.activeIndex - 1 : this.activeIndex + 1
    if (this.activeIndex > 0 && this.activeIndex < list.length + 1) {
      this.wrapper.current.className = 'anim'
      this.wrapper.current.style.transform = `translate3d(${-483 * this.activeIndex}px,0,0)`
      this.setState({activeIndexUI: this.activeIndex - 1})
    } else {
      this.activeIndex = this.activeIndex <= 0 ? list.length + 1 : 0
      this.wrapper.current.removeAttribute('class')
      this.wrapper.current.style.transform = `translate3d(${-483 * this.activeIndex}px,0,0)`
      setTimeout(this.carousel, 0, direction)
    }
  }

  render() {
    const { disableAuth } = this.props
    let {list: uiList, isEnter, activeIndexUI} = this.state
    let len = uiList.length
    if (len > 1) {
      uiList = [uiList[len - 1], ...uiList, uiList[0]]
    }

    if(disableAuth && disableAuth.length > 0 && disableAuth.includes('ad_web.oa')) {
      return <div className="Ad disableAuth">
        <span>目前还没有权限查看哦～</span>
      </div>
    }

    if (uiList.length > 0) {
      return (
        <div
          className="Ad"
          onMouseEnter={this.onMouseEnter}
          onMouseLeave={this.onMouseLeave}>
          <nav
            ref={this.wrapper}
            onTransitionEnd={this.onTransitionEnd}>
            {
              uiList.map((ad, index) => (
                <a
                  key={index}
                  style={{backgroundImage: `url(${ad.picUrl || ''})`}}
                  href={ad.linkUrl || ''}
                  target="_blank"
                ></a>
              ))
            }
          </nav>
          <span
            className={`${isEnter && len > 1 ? '' : 'hide '}left oa-iconfont oa-left-page`}
            onClick={this.prev}
          />
          <span
            className={`${isEnter && len > 1 ? '' : 'hide '}right oa-iconfont oa-right-page`}
            onClick={this.next}
          />
          {
            len > 1
              ?
              <Paging
                no={activeIndexUI}
                count={len}
                onChange={this.toPage}/>
              :
              null
          }
        </div>
      )
    } else {
      return (<div className="Ad">
        <div className="no-ad">
          <div></div>
          糟糕，数据走丢了
        </div>
      </div>)
    }


  }
}

export default Ad
