import React, {Component} from 'react'
import {inject, observer} from 'mobx-react'
import './index.scss'
import Language from '../../language'

@inject('Config')
@observer
class News extends Component{

  state = {
    firstNews : {},
    otherNews: [],
    news: []
  }

  componentWillMount() {
    this.getTracks()
  }

  getTracks = ()=>{
    this.props.httpAgent.get('/cms/hotpot/list/pc').then(res => {
      if(res && res.resultCode === '000000' && res.result && res.result.length > 0) {
        const _other = []
        res.result.map((item, index) => {
          if (index !== 0) {
            _other.push(item)
          }
        })
        this.setState({
          firstNews: res.result[0],
          otherNews: _other,
          news: res.result
        })
      }
    })
  }



  render(){
    const { disableAuth } = this.props
    const LConfig = Language[this.props.Config.language]['News']
    const {firstNews, otherNews, news} = this.state

    if(disableAuth && disableAuth.length > 0 && disableAuth.includes('oa_web.cms')) {
      return <div className="News disableAuth">
        <span>目前还没有权限查看哦～</span>
      </div>
    }

    if (news.length > 0) {
      return (
          <div className="News">
            <header className="s-bg-panel-title s-b-default s-secondary">{LConfig['TITLE']}</header>
            <section>
              <a href={firstNews.url} target="_blank">
                <header>
                  <span className="img" style={{backgroundImage: `url(${firstNews.cover})`}} />
                  <div>
                    <h1 className="s-link">{firstNews.title}</h1>
                    <p className="s-default">
                      {firstNews.description && firstNews.description.substring(0, 30)}...
                      <sub>来源：{firstNews.from}</sub>
                    </p>
                  </div>
                </header>
              </a>
              <ul className="s-b-default s-default">
                {
                  otherNews.length > 0 ? otherNews.map((item, index) => {
                    return <li key={index} onClick={() => {window.open(item.url)}}><a>{item.title}</a><sub>来源：{item.from}</sub></li>
                  }): null
                }
              </ul>
            </section>
          </div>
      )
    } else {
      return (
          <div className="News">
            <header className="s-bg-panel-title s-b-default s-secondary">{LConfig['TITLE']}</header>
            <div className="no-news">
              <div></div>
              糟糕，数据走丢了
            </div>
          </div>
      )
    }

  }
}

export default News
