import React, {Component} from 'react'
import {inject, observer} from 'mobx-react'
import './index.scss'
import Language from '../../language'
import Modal from './Modal'
import {Confirm} from '../common'
import {Tost} from 'karaui'
import setTitleElps from '../../util/setElps'

@inject('Config')
@observer
class Schedule extends Component{
  static PAGE_SIZE = 4

  constructor(props){
    super(props)
    let date = new Date()
    this.state = {
      year: date.getFullYear(),
      month: date.getMonth()+1,
      today: {
        y: date.getFullYear(),
        m: date.getMonth()+1,
        d: date.getDate(),
      },
      marks: [],
      pageNo: 1,
      pageCount: 0,
      result: [],
      operate: null,
      paramDay: null,
      isToday: true,
      now: '',
      showEdit: false,
      showId: '',
    }
    this.show = React.createRef()
    this.mult = React.createRef()
  }

  componentDidMount(){
    const nowDate = new Date()
    const now = `${nowDate.getFullYear()}${nowDate.getMonth() + 1 > 9 ? nowDate.getMonth() + 1 : `0${nowDate.getMonth() + 1}`}${nowDate.getDate() > 9 ? nowDate.getDate() : `0${nowDate.getDate()}`}`
    this.refresh()
    this.setState({now: now})
  }

  componentWillReceiveProps(next) {
    if (next.Config.isSaveSuccess) {
      this.refresh();
    }
  }

  refresh = ()=>{
    const {year, month, today} = this.state
    this.getMonthList(year, month)
    this.getDayList(today.y, today.m, today.d)
  }

  getMonthList(year, month){
    this.props.httpAgent.post('/schedule/event/getList.month', {
      type: 'bot',
      day: `${year}${month<10 ? '0'+month : month}`,
    }).then(res=>res.resultList).then(res=>{
      this.setState({
        marks: res.map(o=>{
          let dd = new Date(o.beginTime)
          return `${dd.getFullYear()}/${dd.getMonth()+1}/${dd.getDate()}`
        })
      })
    })
  }

  getDayList(y, m, d){
    this.props.httpAgent.post('/schedule/event/getList.day/bot', {
      day: `${y}${m<10 ? '0'+m : m}${d<10 ? '0'+d : d}`,
    }).then(res=>{
      res.resultList.forEach(o=>{
        let dd = new Date(o.beginTime)
        let h = dd.getHours()
        let m = dd.getMinutes()
        o.beginTimeShow = `${h<10 ? '0'+h : h}:${m<10 ? '0'+m : m}`
      })
      let newArr = res.resultList
      newArr.map((item, i) => {
        if (item.isAllDay === 'yes') {
          var allDay = newArr.splice(i, 1)
          console.log('allday',allDay);
          newArr.unshift(allDay[0])
        }
      })
      this.setState({
        result: newArr,
        pageNo: 1,
        pageCount: Math.ceil(res.resultList.length/Schedule.PAGE_SIZE),
      })
    })
  }

  generateDatePanel = (year, month)=>{
    let DAY = 24*60*60*1000
    let firstDay = new Date(year, month-1, 1)
    let diffDays = firstDay.getDay() || 7
    firstDay = new Date(firstDay.getTime() - diffDays*DAY)
    let result = []
    for(let i=0; i<42; i++){
      let date = new Date(firstDay.getTime() + i*DAY)
      result.push({
        y: date.getFullYear(),
        m: date.getMonth()+1,
        d: date.getDate(),
      })
    }
    return result
  }

  prev = ()=>{
    let {year, month} = this.state
    year = month===1 ? year-1 : year
    month = month===1 ? 12 : month-1
    this.getMonthList(year, month)
    this.setState({year, month})
  }

  next = ()=>{
    let {year, month} = this.state
    year = month===12 ? year+1 : year
    month = month===12 ? 1 : month+1
    this.getMonthList(year, month)
    this.setState({year, month})
  }

  setToday = e =>{
    if(e.target.title){
      const {year, month, now} = this.state
      let ymd = e.target.title.split('/')
      let y = Number(ymd[0])
      let m = Number(ymd[1])
      let d = Number(ymd[2])
      this.getDayList(y, m, d)
      const paramDay = `${y}${m > 9 ? m : `0${m}`}${d > 9 ? d : `0${d}`}`
      if (paramDay !== now) {
        this.setState({ isToday: false })
      } else {
        this.setState({ isToday: true })
      }
      if(year===y && month===m){
        this.setState({
          today: {y, m, d},
          paramDay
        })
      }else{
        this.getMonthList(y, m)
        this.setState({
          year: y,
          month: m,
          today: {y, m, d},
          paramDay
        })
      }
    }
  }

  prevPage = e=>{
    if(e.target.classList.contains('disabled')) return
    this.setState({pageNo: this.state.pageNo-1})
  }

  nextPage = e=>{
    if(e.target.classList.contains('disabled')) return
    this.setState({pageNo: this.state.pageNo+1})
  }

  modify = event =>{
    if(event.repeatType){
      // 重复事件
      this.setState({
        operate: {
          method: 'modify',
          event,
        }
      })
    }else{
      // 非重复修改事件
      this.props.editCalendar(event.id, false)
    }
  }

  deleteEvent = id=>{
    this.props.httpAgent.post(`/schedule/event/delete/${id}`).then(res=>{
      console.log('del 删除', res);
      if (res && res.resultCode ==='000000') {
        Tost({type:'success', msg: '删除成功', time: 1.5})
        this.refresh()
      } else {
        Tost({type:'error', msg: res.resultMessage || '删除失败'})
      }
    })
  }

  delete = event =>{
    if(event.repeatType){
      this.setState({
        operate: {
          method: 'delete',
          event,
        }
      })
    }else{
      Confirm('删除后无法恢复，是否确认删除？', ()=>{
        console.log('shanchu', event.id)
        this.deleteEvent(event.id)
      })
    }
  }

  onCancel = ()=>{
    this.setState({operate: null})
  }

  deleteRepeatEvent = body=>{
    this.props.httpAgent.post('/schedule/event/modify.repeat', body).then(res=>{
      Tost({type:'success', msg: '删除成功', time: 1.5})
      this.refresh()
    })
  }

  onOK = type =>{
    // 编辑 删除重复事件按钮
    const {operate} = this.state
    if(operate.method === 'delete'){
      let dd = new Date(operate.event.beginTime)
      let y = dd.getFullYear()
      let m = dd.getMonth()+1
      let d = dd.getDate()
      this.deleteRepeatEvent({
        method: operate.method,
        type,
        changeDate: `${y}${m<10 ? '0'+m : m}${d<10 ? '0'+d : d}`,
        scheduleEvent: {
          correlationEventId: operate.event.correlationEventId,
          id: operate.event.id,
        }
      })
    }else if(operate.method === 'modify'){
      // TODO 修改重复事件
      this.props.editCalendar(operate.event.id, true, type)
    }
    this.setState({operate: null})
  }

  // 查看详情
  scheduleDetail =(id) => {
    this.props.calendarShow(id)
  }

  render(){
    const LConfig = Language[this.props.Config.language]['Schedule']
    const {year, month, today, marks, pageNo, pageCount, result, operate} = this.state
    const dp = this.generateDatePanel(year, month)
    return (
      <div className="Schedule">
        <header className="s-bg-panel-title s-b-default s-secondary">
          {LConfig['TITLE']}
          <button
            className="oa-iconfont s-b-default oa-more1 s-primary"
            onClick={() => {this.props.showMonthCal('show')}}
          >
          </button>
        </header>
        <section>
          <div className="left">
            <h1 className="s-default">
              <i onClick={this.prev} className="oa-iconfont oa-left-page"></i>
              &ensp;{year}{LConfig['YEAR_LABEL']}{month}{LConfig['MONTH_LABEL']}&ensp;
              <i onClick={this.next} className="oa-iconfont oa-right-page"></i>
              <i style={{display: 'none'}}>{this.props.Config.isSaveSuccess}</i>
            </h1>
            <header className="s-tips">
              <b>{LConfig['WEEK_SUNDAY']}</b>
              <b>{LConfig['WEEK_MONDAY']}</b>
              <b>{LConfig['WEEK_TUESDAY']}</b>
              <b>{LConfig['WEEK_WEDNESDAY']}</b>
              <b>{LConfig['WEEK_THURSDAY']}</b>
              <b>{LConfig['WEEK_FRIDAY']}</b>
              <b>{LConfig['WEEK_SATURDAY']}</b>
            </header>
            <div onClick={this.setToday}>
              {
                dp.map((date, index)=>{
                  let classNames = []
                  const paramNow = `${date['y']}${date['m'] > 9 ? date['m']  : `0${date['m']}`}${date['d'] > 9 ? date['d'] : `0${date['d']}`}`
                  if(date.y!==year || date.m!==month){
                    classNames.push('gray')
                  }
                  if(date.y===today.y && date.m===today.m && date.d===today.d){
                    classNames.push('cur')
                    classNames.push('s-bg-primary')
                    classNames.push('small')
                  }
                  if (paramNow === this.state.now && !this.state.isToday) {
                    classNames.push('toDay')
                  }
                  if(marks.some(o=>o===`${date.y}/${date.m}/${date.d}`)){
                    classNames.push('mark')
                  }
                  return (
                    <span 
                      key={index} 
                      title={`${date.y}/${date.m}/${date.d}`}
                      className={`s-secondary ${classNames.join(' ')}`}
                    >
                      {date.d}
                    </span>
                  )
                })
              }
            </div>
          </div>
          <div className="right">
            <h1 className="s-default">
                <button
                  className="s-primary"
                  onClick={() => {this.props.calendarShow('', this.state.paramDay)}}
                >
                  +{LConfig['ADD_SCHEDULE']}
                </button>
            </h1>
            <section>
              <ul className="s-default">
                {
                  result && result.length !== 0 ?
                    result.slice((pageNo-1)*Schedule.PAGE_SIZE, pageNo*Schedule.PAGE_SIZE).map(event=> {
                      return <li
                        key={event.id}
                        onMouseEnter={(e) => {
                          e.preventDefault();
                          e.stopPropagation();
                          this.setState({showEdit: true, showId: event.id})
                        }}
                        onMouseLeave={() => {
                          this.setState({showEdit: false})
                        }}
                      >
                        <label
                          className="s-primary">{event.isAllDay === 'yes' ? LConfig['ALL_DAY'] : event.beginTimeShow}
                        </label>
                        <p>
                          {setTitleElps(event.content, 211, 14, 2)}
                        </p>
                        {this.state.showEdit && this.state.showId === event.id ?
                          <div className="pop s-b-default">
                            <header>
                              <label
                                className="s-primary">{event.isAllDay === 'yes' ? LConfig['ALL_DAY'] : event.beginTimeShow}</label>
                              <p>{event.content}</p>
                            </header>
                            <footer>
                              <button
                                className="btn-info buttonDetail"
                                onClick={() => {
                                  this.scheduleDetail(event.id)
                                }}
                              >
                                {LConfig['TO_DETAIL']}
                              </button>
                              {
                                event.isCreator === '1' ?
                                  <button
                                    className="s-info"
                                    onClick={e => {
                                      this.modify(event)
                                    }}
                                  >
                                    {LConfig['TO_EDIT']}
                                  </button> : null
                              }
                              <button
                                className="s-danger"
                                onClick={e => {
                                  this.delete(event)
                                }}
                              >
                                {LConfig['TO_DEL']}
                              </button>
                            </footer>
                          </div> : null
                        }
                      </li>
                    }) :
                    <p>
                      <span className='noCalendarImg'/>
                      <span className='noCalendarText text-grey'>别让时光匆匆流过</span>
                    </p>
                }
              </ul>
              {
                pageCount>1
                ?
                <div className="paging">
                  <span 
                    onClick={this.prevPage} 
                    className={`s-bg-page oa-iconfont oa-left-page ${pageNo<=1 ? 'disabled s-tips' : 's-info'}`}/>
                  {pageNo}
                  <span 
                    onClick={this.nextPage} 
                    className={`s-bg-page oa-iconfont oa-right-page ${pageNo>=pageCount ? 'disabled s-tips' : 's-info'}`}/>
                </div>
                :
                null
              }
            </section>
          </div>
        </section>
        {
          operate
          ?
          <Modal 
            method={operate.method}
            onCancel={this.onCancel}
            onOK={this.onOK} />
          :
          null
        }
      </div>
    )
  }
}

export default Schedule
