import React, {Component} from 'react'
import {inject, observer} from 'mobx-react'
import './index.scss'
import Pop from './Pop'
import Language from '../../language'
import {Scroll, Tost} from 'karaui'

@inject('Config')
@observer
class Todo extends Component{
  static PAGE_SIZE = 17

  state = {
    type: 'todo',
    todoInfo: {
      pageNo: 1,
      count: 0,
      result: [],
    },
    trackInfo: {
      pageNo: 1,
      count: 0,
      result: [],
    },
    otherTrackInfo: {
      pageNo: 1,
      count: 0,
      result: [],
    },
    todoActive: null,
    trackActive: null,
    pop: null,
  }

  componentDidMount(){
    document.addEventListener('mousedown', (e)=>{
      if(e.target.parentNode.classList && e.target.parentNode.classList[0] === 'horizontal-bar') {
        // 水平滚动条 点击不隐藏内容 不能用点击事件 点击的目标是鼠标松开 但是鼠标松开一般都会离开目标元素
        this.isHorizontalBar = true
        e.stopPropagation()
        return false
      }
      this.isHorizontalBar = false
    })
    document.addEventListener('click', (e)=>{
      if(this.isHorizontalBar) {
        // 水平滚动条 点击不隐藏内容 不能用点击事件 点击的目标是鼠标松开 但是鼠标松开一般都会离开目标元素
        e.stopPropagation()
        return false
      }
      this.setState({
        todoActive: null,
        trackActive: null,
        pop: null
      })
    }, false)
    this.refresh()
  }

  refresh = ()=>{
    this.getTodos(1)
    this.getTracks(1)
    this.getOtherTasktrace(1)
  }

  getTodos = pageNo=>{
    // 代办
    this.props.httpAgent.post('/task/taskinfo/query', {
      pageStart: (pageNo-1)*Todo.PAGE_SIZE+1,
      pageEnd: pageNo*Todo.PAGE_SIZE,
    }).then(res=>{
      this.setState({
        todoInfo: {
          pageNo,
          count: res.count,
          result: res.taskInfo||[],
        }
      })
    })
  }

  getTracks = pageNo=>{
    this.props.httpAgent.post('/task/tasktrace/query',{
      pageStart: (pageNo-1)*Todo.PAGE_SIZE+1,
      pageEnd: pageNo*Todo.PAGE_SIZE,
    }).then(res=>{
      this.setState({
        trackInfo: {
          pageNo,
          count: res.count,
          result: res.taskTraceInfo||[],
        }
      })
    })
  }

  getOtherTasktrace = pageNo => {
    this.props.httpAgent.get('/task/taskinfo/otherTask').then(res=>{
      this.setState({
        otherTrackInfo: {
          pageNo,
          count: res.count || 0,
          result: res.details||[],
        }
      })
    })
  }

  prev = ()=>{
    const {type, todoInfo, trackInfo} = this.state
    if(type==='todo'){
      this.getTodos(todoInfo.pageNo-1)
    }else if(type==='track'){
      this.getTracks(trackInfo.pageNo-1)
    }
  }

  next = ()=>{
    const {type, todoInfo, trackInfo} = this.state
    if(type==='todo'){
      this.getTodos(todoInfo.pageNo+1)
    }else if(type==='track'){
      this.getTracks(trackInfo.pageNo+1)
    }
  }

  todoAgree = ()=>{
    this.postTodo()
  }

  todoReject = ()=>{
    const {todoActive} = this.state
    if(!todoActive.rejectReason){
      Tost({type:'error',msg: '请输入驳回原因', time: 1.5})
    }else{
      this.postTodo(false)
    }
  }

  postTodo = (isAgree=true)=>{
    const {todoActive} = this.state
    this.props.httpAgent.post('/task/taskinfo/async', {
      systemId: todoActive.systemId,
      targetTaskId: todoActive.targetTaskId,
      service: isAgree ? todoActive.acceptBtnUrl : todoActive.rejectBtnUrl,
      sync: '0',
      channel: 'oa',
      filter: 1,
      rejectReason: isAgree ? undefined : todoActive.rejectReason,
    }).then(res=>{
      Tost({type: 'success',msg: '审批成功', time: 1.5})
      this.setState({todoActive: null})
      this.refresh()
    })
  }

  stopPropagation = e=>{
    e.nativeEvent.stopImmediatePropagation()
    this.setState({pop: null})
  }

  setType = (type) =>{
    if(type){
      this.setState({type})
    }
  }

  showApprove = (e, todoActive)=>{
    if(!this.state.todoActive){
      e.nativeEvent.stopImmediatePropagation()
      this.setState({
        todoActive: {
          ...todoActive,
          isApproveReject: false,
          rejectReason: '',
        }
      })
    }
  }

  toggleReject = e=>{
    const {todoActive} = this.state
    this.setState({
      todoActive: {
        ...todoActive,
        isApproveReject: !todoActive.isApproveReject,
      }
    })
  }

  showTrack = (e, trackActive)=>{
    if(!this.state.trackActive){
      e.nativeEvent.stopImmediatePropagation()
      this.setState({trackActive})
    }
  }

  showPop = (e, node)=>{
    e.nativeEvent.stopImmediatePropagation()
    e.stopPropagation()
    if (!node.accInfo || node.nodeSeq === "1") {
      return false
    }
    let rect = e.target.getBoundingClientRect()
    let appPos = document.getElementById('app').getBoundingClientRect()
    const {pop} = this.state
    if(pop && pop.node.nodeSeq===node.nodeSeq){
      this.setState({pop: null})
    }else{
      this.setState({
        pop: {
          node,
          left: rect.left - appPos.left - 23,
          top: rect.top +24,
        }
      })
    }
  }

  // 好像没有什么用
  stopTrackPropagation = e=>{
    e.nativeEvent.stopImmediatePropagation()
    // if(this.state.pop){
    this.setState({trackActive: null, pop: null})
    // }
  }

  rejectReasonChange = e=>{
    const {todoActive} = this.state
    this.setState({
      todoActive: {
        ...todoActive,
        rejectReason: e.target.value,
      }
    })
  }

  render(){
    console.log('render',this.state)
    const { disableAuth } = this.props
    const LConfig = Language[this.props.Config.language]['Todo']
    const {type, todoInfo, todoActive, trackInfo, otherTrackInfo, trackActive, pop} = this.state
    let isZH = this.props.Config.language==='zh'
    let dataPanel = null
    let pageInfo = {}
    let isNothing = false
    if(type ==='todo'){
      isNothing = todoInfo.count===0
      pageInfo = todoInfo
      dataPanel = (
          <ul className="todoList">
            {
              todoInfo.result.map((todo, index)=>(
                  <li onClick={e=>this.showApprove(e, todo)} key={todo.id}>
                    <label>{(todoInfo.pageNo-1)*Todo.PAGE_SIZE+index+1}、{isZH ? todo.titleInfo.taskTitleCn : todo.titleInfo.taskTitleEn}</label>
                    <p>{isZH ? todo.titleInfo.taskDescCn : todo.titleInfo.taskDescEn}</p>
                  </li>
              ))
            }
          </ul>
      )
    }else if(type === 'track'){
      isNothing = trackInfo.count===0
      pageInfo = trackInfo
      dataPanel = (
          <ul className="trackList">
            {
              trackInfo.result.map((track, index)=>(
                  <li onClick={e=>this.showTrack(e, track)} key={track.orderId}>
                    <label>{(trackInfo.pageNo-1)*Todo.PAGE_SIZE+index+1}、{isZH ? track.titleInfo.orderTitleCn : track.titleInfo.orderTitleEn}</label>
                    <p>{isZH ? track.titleInfo.orderDescCn : track.titleInfo.orderDescEn}</p>
                    <span className="s-primary">{track.isStop !== '0' ? LConfig['TRACK_PROCESSEND'] : LConfig['TRACK_PROCESSING']}</span>
                    <button className="s-bg-primary">{LConfig['TRACK_BUTTON_PROCESS']}</button>
                  </li>
              ))
            }
          </ul>
      )
    }else if(type === 'other'){
      pageInfo = {pageNo: 1, count: 0}
      dataPanel = (
          <div className="other-info">
            {
              otherTrackInfo && otherTrackInfo.result.map((item) => {
                return <div className="other-info-item">{LConfig[item.order]}: <a className="s-info" href={item.url} target="_blank">{item.count}</a></div>
              })
            }
          </div>
      )
    }

    if(disableAuth && disableAuth.length > 0 && disableAuth.includes('oa_web.task')) {
      return <div className="Todo disableAuth">
        <span>目前还没有权限查看哦～</span>
      </div>
    }

    return (
        <div className={`Todo ${isNothing ? 'nothing' : 'has'}`}>
          <ol className="s-bg-panel-title">
            <li
                onClick={() => this.setType('todo')}
                className={`s-secondary s-b-default ${type==='todo' ? 'active' : ''}`}>
              {LConfig['TODO_TITLE']}&ensp;
              <span className="red">{todoInfo.count}</span>
            </li>
            <li
                onClick={() => this.setType('track')}
                className={`s-secondary s-b-default ${type==='track' ? 'active' : ''}`}>
              {LConfig['TRACK_TITLE']}&ensp;
              <span className="blue">{trackInfo.count}</span>
            </li>
            <li
                onClick={() => this.setType('other')}
                className={`s-secondary s-b-default ${type==='other' ? 'active' : ''}`}>
              {LConfig['OTHER_TITLE']}&ensp;
              <span className="green">{otherTrackInfo.count}</span>
            </li>
            <li className="s-secondary s-b-default empty"></li>
            <li onClick={this.refresh} className="oa-iconfont s-primary s-b-default oa-refresh"/>
            <li onClick={() => window.open(this.props.todoMoreUrl)} className="s-primary oa-iconfont s-b-default oa-more1"/>
          </ol>
          <section>
            {
              isNothing
                  ?
                  <p className="nothing s-secondary">
                    <b></b>
                    &emsp;当前还没有待办内容~
                  </p>
                  :
                  null
            }
            {
              dataPanel
            }
            {
              pageInfo.pageNo>1
                  ?
                  <span onClick={this.prev} className='left icon oa-iconfont oa-left-page' />
                  :
                  null
            }
            {
              pageInfo.pageNo*Todo.PAGE_SIZE<pageInfo.count
                  ?
                  <span onClick={this.next} className='right icon oa-iconfont oa-right-page' />
                  :
                  null
            }
          </section>
          {
            todoActive
                ?
                <div
                    className="approve s-b-default"
                    onClick={this.stopPropagation}>
                  <header>
                    <label>{(todoInfo.pageNo-1)*Todo.PAGE_SIZE+todoInfo.result.findIndex(todo=>todo.id===todoActive.id)+1}、{isZH ? todoActive.titleInfo.taskTitleCn : todoActive.titleInfo.taskTitleEn}</label>
                    <p>{isZH ? todoActive.titleInfo.taskDescCn : todoActive.titleInfo.taskDescEn}</p>
                  </header>
                  <h2>
                    {
                      todoActive.acceptBtnUrl
                          ?
                          <button className="s-bg-primary small" onClick={this.todoAgree}>{LConfig['TODO_AGREE']}</button>
                          :
                          null
                    }
                    {
                      todoActive.rejectBtnUrl
                          ?
                          <button className="s-bg-slider-dot" onClick={this.toggleReject}>{LConfig['TODO_REJECT']}</button>
                          :
                          null
                    }
                    {
                      todoActive.viewBtnUrl
                          ?
                          <a className="btn-info" href={todoActive.viewBtnUrl} target="_blank">{LConfig['TODO_DETAIL']}</a>
                          :
                          null
                    }
                    <span className="s-secondary">{todoActive.taskDispTime.substr(0, 10)}</span>
                  </h2>
                  {
                    todoActive.isApproveReject
                        ?
                        <textarea
                            className="s-b-default"
                            value={todoActive.rejectReason}
                            onChange={this.rejectReasonChange}
                            placeholder={LConfig['TODO_REJECT_PLACEHOLDER']}></textarea>
                        :
                        null
                  }
                  {
                    todoActive.isApproveReject
                        ?
                        <footer>
                          <button className="s-bg-primary small" onClick={this.todoReject}>{LConfig['TODO_SUBMIT']}</button>
                        </footer>
                        :
                        null
                  }
                </div>
                :
                null
          }
          {
            trackActive
                ?
                <div
                    className="track s-b-default">
                  <header>
                    <label>{(trackInfo.pageNo-1)*Todo.PAGE_SIZE+trackInfo.result.findIndex(track=>track.orderId===trackActive.orderId)+1}、{isZH ? trackActive.titleInfo.orderTitleCn : trackActive.titleInfo.orderTitleEn}</label>
                    <p>{isZH ? trackActive.titleInfo.orderDescCn : trackActive.titleInfo.orderDescEn}</p>
                    <span className="s-primary s-min-text">{trackActive.isStop !== '0' ? LConfig['TRACK_PROCESSEND'] : LConfig['TRACK_PROCESSING']}</span>
                    <button className="s-bg-primary s-min-text">{LConfig['TRACK_BUTTON_PROCESS']}</button>
                  </header>
                  <div className="trackScroll">
                    <Scroll>
                      <ul>
                        {
                          trackActive.nodeInfo.map(node=>{
                            let liClass = ''
                            let iClass = ''
                            // isStop 0 未结束 1 正常结束 2 撤销结束
                            // nodeStatus 0 未处理 1 已处理 2 处理中
                            if(trackActive.isStop !== '0' || node.nodeStatus === '1') {
                              liClass = 's-secondary s-b-secondary-text done'
                              iClass = 's-bg-slider-dot'
                            }else if(node.nodeStatus === '2') {
                              liClass = 's-b-primary s-primary wait'
                              iClass = 's-b-primary s-bg-primary small'
                            }else{
                              liClass = 's-b-primary s-primary'
                              iClass = 's-b-primary iDefault'
                            }
                            return <li className={liClass} key={node.nodeSeq}>
                            <i className={iClass} onClick={e => this.showPop(e, node)}></i>
                            <p>
                            {node.accountName || node.accInfo && node.accInfo.map((item, index) => {
                              return item.accountName + (index !== node.accInfo.length - 1 ? '、' : ' ')
                            }) || (isZH ? node.nodeNameCn : node.nodeNameEn)}
                            <br/>{node.nodeStatusDate}</p>
                            </li>
                        })
                        }
                      </ul>
                    </Scroll>
                  </div>
                </div>
                :
                null
          }
          {
            pop
                ?
                <Pop node={pop.node} left={pop.left} top={pop.top} {...this.props}/>
                :
                null
          }
        </div>
    )
  }
}

export default Todo
