import React, { Component } from "react";
import { inject, observer } from "mobx-react";
import "./index.scss";
import { Paging } from "../common";
import Language from "../../language";
import Modal from "./Modal";
import { Tost, Modal as KarauiModal } from "karaui";
import NotLan from "../NotLan";
import loadImage from "../NotLan/loadImage";

@inject("Config")
@observer
class Tool extends Component {
  static COMMON_ID = "wc_custom";
  static PAGE_SIZE = 10;

  state = {
    types: [],
    typeID: Tool.COMMON_ID,
    nodeTree: {
      [Tool.COMMON_ID]: [],
    },
    webAppInfos: {},
    pageNo: 0,
    isEdit: false,
    isModal: false,
    customWebAppList: [],
    dragCustomId: "",
    notLanModal: false,
  };

  constructor(props) {
    super(props);
    this.dragCustomPageTime = 0;
  }

  componentDidMount() {
    this.props.httpAgent.get("/webhook/w_apps.get").then((res) => {
      this.setState({
        types: [
          {
            id: Tool.COMMON_ID,
            name: Language["zh"]["Tool"]["TYPE_COMMON"],
            nameEn: Language["en"]["Tool"]["TYPE_COMMON"],
          },
          ...res.appCatalogInfos,
        ],
        nodeTree: {
          ...res.nodeTree,
          [Tool.COMMON_ID]: res.customWebAppList.map((id) => ({ id })),
        },
        customWebAppList: res.customWebAppList,
        webAppInfos: res.webAppInfos,
      });
    });
  }

  setTypeID = (typeID) => {
    this.setState({
      typeID,
      pageNo: 0,
    });
  };

  onDragStart = (dragID, e) => {
    this.originIDs = this.state.types.map((type) => type.id).join("#");
    this.dragID = dragID;
    this.dragCustomId = "";
    e.dataTransfer.setData("Text", dragID);
  };

  onDragEnter = (endID, e) => {
    e.preventDefault();
    if (!this.dragID || endID === Tool.COMMON_ID || endID === this.dragID)
      return;

    let types = this.state.types;
    let start = types.findIndex((type) => type.id === this.dragID);
    let end = types.findIndex((type) => type.id === endID);
    let startItem = types.splice(start, 1)[0];
    let pos = types.findIndex((type) => type.id === endID);
    if (start < end) {
      pos++;
    }
    types.splice(pos, 0, startItem);
    this.setState({ types });
  };

  onDragOver = (e) => {
    e.preventDefault();
  };

  onDragEnd = (e) => {
    e.preventDefault();
    if (!this.dragID) return;
    const destIDs = this.state.types.map((type) => type.id).join("#");
    if (this.originIDs !== destIDs) {
      this.saveTypes(
        this.state.types
          .filter((type) => type.id !== Tool.COMMON_ID)
          .map((type) => type.id)
      );
    }
    this.dragID = "";
  };

  onCustomDragStart = (dragCustomId, e) => {
    this.dragCustomId = dragCustomId;
    this.dragID = "";
    e.dataTransfer.setData("Text", e.target.id);
  };

  onCustomDragEnter = (targetCustomId) => {
    if (!this.dragCustomId) {
      return;
    }
    let customWebAppList = this.state.customWebAppList;
    if (targetCustomId !== this.dragCustomId && targetCustomId !== "add") {
      const targetIdx = customWebAppList.indexOf(targetCustomId);
      if (targetIdx >= 0) {
        const dragIdx = customWebAppList.indexOf(this.dragCustomId);
        if (dragIdx >= 0) {
          customWebAppList.splice(dragIdx, 1);
        }
        customWebAppList.splice(targetIdx, 0, this.dragCustomId);
        this.setState({ customWebAppList });
      }
    }
  };

  onCustomDragEnd = (e) => {
    e.preventDefault();
    const customWebAppList = this.state.customWebAppList;
    this.setState({ customWebAppList });
    this.dragCustomId = "";
  };

  onCustomDragPagePrev = (e) => {
    if (
      !this.dragCustomId ||
      this.dragCustomPageTime > Date.now() - 1000 ||
      this.state.pageNo === 0
    ) {
      return;
    }
    this.dragCustomPageTime = Date.now();
    this.setState({ pageNo: this.state.pageNo - 1 });
  };

  onCustomDragPageNext = (e) => {
    if (!this.dragCustomId || this.dragCustomPageTime > Date.now() - 1000) {
      return;
    }
    this.dragCustomPageTime = Date.now();
    this.setState({ pageNo: this.state.pageNo + 1 });
  };

  saveTypes = (types) => {
    this.props.httpAgent
      .post("/webhook/w_custom_appCatalog.set", {
        webCustomAppCatalogIds: types,
      })
      .then((res) => {
        Tost({ type: "success", msg: "调整应用组顺序成功", time: 1.5 });
      });
  };

  pageChange = (pageNo) => {
    this.setState({ pageNo });
  };

  saveCustom = () => {
    const customIds = this.state.customWebAppList.join("#");
    if (customIds === this.customOriginIds) {
      return;
    }
    this.props.httpAgent
      .post("/webhook/w_custom_apps.set", {
        webCustomAppIds: this.state.customWebAppList,
      })
      .then((res) => {
        Tost({ type: "success", msg: "保存常用应用成功", time: 1.5 });
      });
  };

  toggleEdit = () => {
    if (this.state.isEdit) {
      this.saveCustom();
    } else {
      this.customOriginIds = this.state.customWebAppList.join("#");
    }
    this.setState({
      isEdit: !this.state.isEdit,
    });
  };

  deleteApp = (deleteCustomId, e) => {
    e.preventDefault();
    const customWebAppList = this.state.customWebAppList;
    const targetIdx = customWebAppList.indexOf(deleteCustomId);
    if (targetIdx >= 0) {
      customWebAppList.splice(targetIdx, 1);
      this.setState({ customWebAppList });
    }
  };

  onToggle = (id) => {
    const customWebAppList = this.state.customWebAppList;
    const addIdx = customWebAppList.indexOf(id);
    if (addIdx < 0) {
      customWebAppList.unshift(id);
      this.setState({ customWebAppList });
    } else {
      customWebAppList.splice(addIdx, 1);
      this.setState({ customWebAppList });
    }
  };

  onClose = () => {
    this.setState({ isModal: false });
  };

  showModal = () => {
    this.setState({ isModal: true });
  };

  onOpen = (webAppInfo) => {
    if (webAppInfo && webAppInfo.url) {
      if (webAppInfo.url === "/wxtp") {
        window.$emitter.emit("wxtp");
        return;
      }
      if (webAppInfo.openMode === 1) {
        this.props.history.push(webAppInfo.url);
      } else {
        window.open(webAppInfo.url);
      }
    }
  };

  getDes = (webAppInfo) => {
    if (webAppInfo) {
      const desc =
        this.props.Config.language === "zh"
          ? webAppInfo.appDesc
          : webAppInfo.appDescEn;
      if (desc) {
        return desc.replace(/\\n/gi, "&#10;");
      }
    }
    return "";
  };

  // 最近访问
  recentVisit = (id) => {
    const storage = localStorage.getItem("recentVisit");
    if (!storage) {
      // 无最近浏览
      localStorage.setItem("recentVisit", id);
    } else {
      // 设置缓存
      let idArr = storage.split(",");
      let newStorage = "";
      idArr.unshift(id);
      this.arrSort(idArr).map((i, idx) => {
        if (i && i !== "") {
          newStorage += `${i},`;
        }
      });
      localStorage.setItem("recentVisit", newStorage);
    }
  };

  openLink = (id) => {
    const { isEdit } = this.state;
    if (isEdit) {
      return;
    }
    this.recentVisit(id);
    if (sessionStorage.lan !== "1") {
      // 不在局域网内部
      this.checkNet(id);
      return;
    }
    this.forwardLink(id);
  };

  checkNet = (id) => {
    loadImage("https://xin-apps.asiainfo.com:19683/xin/validate/network").then(
      (image) => {
        sessionStorage.lan = image.height > 0 ? "1" : "0";
        if (sessionStorage.lan !== "1") {
          loadImage("https://sso.asiainfo.com:443/img/lock.png").then(
            (image) => {
              sessionStorage.lan = image.height > 0 ? "1" : "0";
              if (sessionStorage.lan !== "1") {
                this.openNotLanModal();
              } else {
                this.forwardLink(id);
              }
            }
          );
        } else {
          this.forwardLink(id);
        }
      }
    );
  };

  forwardLink = (id) => {
    const { webAppInfos } = this.state;
    if (webAppInfos[id].url === "/wxtp") {
      window.$emitter.emit("wxtp");
      return;
    }
    if (webAppInfos[id].openMode === 1) {
      this.props.history.push(webAppInfos[id].url);
    } else {
      window.open(webAppInfos[id].url);
    }
  };

  arrSort = (arr) => {
    let newArr = [];
    arr.map((i, idx) => {
      if (newArr.indexOf(i) === -1) {
        newArr.push(i);
      }
    });
    return newArr;
  };

  openNotLanModal = () => {
    this.setState({ notLanModal: true });
  };
  closeNotLanModal = () => {
    this.setState({ notLanModal: false });
  };

  render() {
    const { history, disableAuth } = this.props;
    const {
      types,
      typeID,
      nodeTree,
      webAppInfos,
      isEdit,
      isModal,
      customWebAppList,
    } = this.state;
    // const newImgArr = ['wc_operationManagement', 'wc_rd']; // 是否显示角标
    let isZH = this.props.Config.language === "zh";
    const LConfig = Language[this.props.Config.language]["Tool"];
    let isCustom = typeID === Tool.COMMON_ID;
    let uiList = [];
    if (isCustom) {
      uiList = customWebAppList;
      if (isEdit) {
        uiList = ["add", ...uiList];
      }
    } else {
      uiList = nodeTree[typeID].map((app) => app.id);
    }
    //过滤不存在的ID
    uiList = uiList.filter((id) => {
      return !!webAppInfos[id];
    });
    if (isCustom && isEdit) {
      uiList = ["add", ...uiList];
    }
    let count = Math.ceil(uiList.length / Tool.PAGE_SIZE);
    let pageNo = this.state.pageNo;
    if (pageNo > count) {
      pageNo = count - 1;
    }

    if (
      disableAuth &&
      disableAuth.length > 0 &&
      disableAuth.includes("oa_web.apps")
    ) {
      return <div className="Tool" style={{ display: "none" }}></div>;
    }

    return (
      <div className="Tool">
        {!isEdit && isCustom && uiList.length === 0 ? (
          <span className="arrow" />
        ) : null}
        <ol onDragEnd={(e) => this.onDragEnd(e)} className="s-bg-panel-title">
          {types.map((type) => (
            <li
              onClick={() => this.setTypeID(type.id)}
              onDragStart={(e) => this.onDragStart(type.id, e)}
              onDragEnter={(e) => this.onDragEnter(type.id, e)}
              onDragOver={this.onDragOver}
              key={type.id}
              draggable={type.id !== Tool.COMMON_ID}
              className={
                typeID === type.id
                  ? "s-secondary s-success active"
                  : "s-secondary"
              }
            >
              {isZH ? type.name : type.nameEn}
              {/*newImgArr.indexOf(type.id) !== -1 ? <p/> : null*/}
            </li>
          ))}
        </ol>
        <nav
          onDragEnd={(e) => this.onCustomDragEnd(e)}
          style={{ marginLeft: -pageNo * 970 }}
        >
          {!isEdit && isCustom && uiList.length === 0 ? (
            <p>请点击“自定义”添加系统至常用</p>
          ) : (
            uiList.map((id) =>
              id === "add" ? (
                <div key={id} onClick={this.showModal}>
                  <b className="add" />
                  <p>{LConfig["ADD"]}</p>
                </div>
              ) : (
                <div
                  draggable={isCustom && isEdit}
                  title={this.getDes(webAppInfos[id])}
                  onClick={() => {
                    this.openLink(id);
                  }}
                  onDragStart={(e) => this.onCustomDragStart(id, e)}
                  onDragEnter={(e) => this.onCustomDragEnter(id, e)}
                  key={id}
                  data-id={id}
                >
                  <b
                    className="s-bg-primary"
                    style={{ backgroundImage: `url(${webAppInfos[id].icon})` }}
                  >
                    {isEdit && isCustom ? (
                      <i
                        onClick={(e) => this.deleteApp(id, e)}
                        className="oa-iconfont oa-close_small-copy close-modal"
                      ></i>
                    ) : null}
                  </b>
                  <p>
                    {isZH ? webAppInfos[id].appName : webAppInfos[id].appNameEn}
                  </p>
                </div>
              )
            )
          )}
        </nav>
        {isEdit && isCustom && this.dragCustomId ? (
          <span
            onDragOver={this.onCustomDragPagePrev}
            className={`${pageNo > 0 ? "" : "hide "}left kara-oa-font`}
          >
            &#xe618;
          </span>
        ) : null}
        {isEdit && isCustom && this.dragCustomId ? (
          <span
            onDragOver={this.onCustomDragPageNext}
            className={`${pageNo < count - 1 ? "" : "hide "}right kara-oa-font`}
          >
            &#xe7a5;
          </span>
        ) : null}
        <footer>
          <Paging no={pageNo} count={count} onChange={this.pageChange} />
          {isCustom ? (
            <button className="btn-info outline" onClick={this.toggleEdit}>
              {isEdit ? LConfig["BUTTON_OK"] : LConfig["BUTTON_CUSTOM"]}
            </button>
          ) : null}
          {count > 1 && pageNo > 0 ? (
            <i
              className="ko-tool-left-arrow oa-iconfont oa-left-page"
              onClick={() => {
                this.pageChange(pageNo - 1 < 0 ? count - 1 : pageNo - 1);
              }}
            />
          ) : null}
          {count > 1 && pageNo < count - 1 ? (
            <i
              className="ko-tool-right-arrow oa-iconfont oa-right-page"
              onClick={() => {
                this.pageChange(pageNo + 1 >= count ? 0 : pageNo + 1);
              }}
            />
          ) : null}
        </footer>
        {isModal ? (
          <Modal
            {...{
              types: types.filter((type) => type.id !== Tool.COMMON_ID),
              customList: this.state.customWebAppList,
              nodeTree,
              webAppInfos,
              onToggle: this.onToggle,
              onDeleteToggle: this.onToggle,
              onClose: this.onClose,
              httpAgent: this.props.httpAgent,
            }}
          />
        ) : null}
        <KarauiModal
          show={this.state.notLanModal}
          parent={document.getElementById("app") || document.body}
          replaceStyle={{
            top: "40px",
            right: "0",
            bottom: "0",
            left: "70px",
            zIndex: "10",
            width: "auto",
            height: "auto",
            backgroundColor: "#fff",
          }}
          outsideClick={{
            isCall: true,
            callback: this.closeNotLanModal,
          }}
        >
          <NotLan closeNotLanModal={this.closeNotLanModal} />
        </KarauiModal>
      </div>
    );
  }
}

export default Tool;
